.ConfirmModal__Footer{border-top:1px solid #dee6f3;height:64px;display:-webkit-flex;display:flex;-webkit-justify-content:center;justify-content:center;-webkit-align-items:center;align-items:center}.ConfirmModal__Body{padding:25px 20px;display:-webkit-flex;display:flex}.ConfirmModal__Icon{margin-right:30px;font-size:4.5rem}.ConfirmModal__Title{color:#1c2852;padding-left:0;padding-bottom:5px;padding-top:5px}.ConfirmModal__Content .Text{color:#1c2852;max-width:260px}.ConfirmModal__CancelButton{margin-right:10px}.CodeBlock{background:#f2f3f4;border-radius:.375rem;padding:1em 3em 1em 1em;position:relative;min-height:3.5rem;display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center}.CodeBlock pre{margin:0;font-style:normal;font-weight:400;font-size:.875rem;line-height:1.3125em;color:#031123}.CodeBlock .prism-code{background-color:transparent!important}.CodeBlock__copy__span{top:1em;right:1em;width:1.5rem;height:1.5rem;color:#414b6d;display:-webkit-flex;display:flex;cursor:pointer;position:absolute;-webkit-align-items:center;align-items:center;-webkit-justify-content:center;justify-content:center}.BookmarkCard__container{position:relative;display:-webkit-flex;display:flex;-webkit-flex-direction:column;flex-direction:column;border:.0625rem solid #dee6f3;border-radius:.5rem;padding:1.5rem;margin-bottom:1rem}.BookmarkCard__delete{margin-left:.25rem}.BookmarkCard__top{-webkit-flex:1 1;flex:1 1}.BookmarkCard__top p{margin-top:.25rem;font-weight:400;font-size:.75rem}.BookmarkCard__title__section{display:-webkit-flex;display:flex;-webkit-justify-content:space-between;justify-content:space-between}.BookmarkCard__title__section a{text-decoration:none}.BookmarkCard__title{font-weight:600;font-size:18px}.BookmarkCard__bottom{margin-top:.75rem}.BookmarkCard__bottom strong{color:#243969;margin-right:.5rem}.BookmarkCard__selected__metrics{margin-top:1rem;width:100%;white-space:nowrap;overflow:auto}.BookmarkCard__selected__metrics .Badge{margin-right:.5rem}.BookmarkCard__run__expression{margin-right:2.8125rem}.AppBar__container{display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center;min-height:2.3rem;border-bottom:.0625rem solid #dceafb;border-left:.0625rem solid #e8f1fc;border-right:.0625rem solid #e8f1fc;padding-left:1.5rem}.AppBar__title{-webkit-flex:1 1;flex:1 1;text-transform:uppercase;letter-spacing:.07em}.AppBar__content{display:-webkit-flex;display:flex;-webkit-align-items:center;align-items:center}.Bookmarks{height:100vh}.Bookmarks__title{margin:.5em 0;padding:0;color:#1473e6}.Bookmarks__list{padding:1.5em}.BusyLoaderWrapper,.EmptyComponent{display:-webkit-flex;display:flex;-webkit-justify-content:center;justify-content:center;-webkit-align-items:center;align-items:center}.EmptyComponent{width:100%;height:calc(100% - 40px);-webkit-flex-direction:column;flex-direction:column}.EmptyComponent__small__title{font-weight:600;font-size:.875rem}.EmptyComponent__small__content{font-weight:500;font-size:.875rem}.EmptyComponent__small__img{padding:10px}.EmptyComponent__small__img img{width:50px;height:57px}.EmptyComponent__medium__title{font-weight:600;font-size:.875rem}.EmptyComponent__medium__content{font-weight:500;font-size:.875rem}.EmptyComponent__medium__img{padding:10px}.EmptyComponent__medium__img img{width:70px;height:97px}.EmptyComponent__big__content,.EmptyComponent__big__title{font-weight:500;font-size:1rem}.EmptyComponent__big__img{padding:10px}.EmptyComponent__big__img img{width:90px;height:110px}
/*# sourceMappingURL=16.a8fcff8c.chunk.css.map */